// BtlShp1.cpp
// add to BtlShp0.cpp a BigShip class
// tcc BtlShp1.cpp

#include <stdlib.h> // srand() time()

#include "stdafx.h"
#include "BtlShp1.h"
#include "BattleShip01.h"
#include "BattleShip01Dlg.h"

//-------------------------------------------------------------------------

// random number generator, constructed without any parameter
// initialize the random number with the time function
// each call to next will return an integer between low and high

Ship::Ship(int length, int init_x, int init_y, int xdir, int ydir)
     : m_length(length), m_sunk(0)
{
    for (int i=0; i<length; i++)
    {
        m_xpos[i] = init_x + xdir * i;
        m_ypos[i] = init_y + ydir * i;
    }
    m_hIconCross = AfxGetApp()->LoadIcon(IDI_ICON_CROSS);
    m_hIconPound = AfxGetApp()->LoadIcon(IDI_ICON_POUND);
}

//-------------------------------------------------------------------------

int Ship::hit(int xtarget, int ytarget)
{
    if (m_sunk == 1) return 0;
    for (int i=0; i<m_length; i++)
        if ((m_xpos[i]==xtarget)&&(m_ypos[i]==ytarget))
        {
            m_sunk = 1;
            m_xpos[i] = m_xpos[i] + 100; // mark the position
            m_ypos[i] = m_ypos[i] + 100;
            return 1;
        }
    return 0;
}

int Ship::isSunked()
{
    if (m_sunk == 1)
        return 1;
    else
        return 0;
}

void Ship::draw(CDC *pdc, CBattleShip01Dlg *pBoard)
{
    if (m_sunk == 1)
        for (int i=0; i<m_length; i++)
        {
            if ((m_xpos[i]<100)&&(m_ypos[i]<100)) // not hit
                pBoard->drawSymbol(pdc,
                    m_hIconPound, m_xpos[i], m_ypos[i]);
            else
                pBoard->drawSymbol(pdc,
                    m_hIconCross, m_xpos[i]-100, m_ypos[i]-100);
        }
}

int Ship::getTouched(Ship ship)
{
    int iFirst, iSecond;

    for (iFirst=0; iFirst<m_length; iFirst++)
        for (iSecond=0; iSecond<ship.m_length; iSecond++)
            if ( ((m_xpos[iFirst]-1 == ship.m_xpos[iSecond]) &&
                  (m_ypos[iFirst] == ship.m_ypos[iSecond]))     ||
                 ((m_xpos[iFirst] == ship.m_xpos[iSecond]) &&
                  (m_ypos[iFirst]-1 == ship.m_ypos[iSecond]))     ||
                 ((m_xpos[iFirst] == ship.m_xpos[iSecond]) &&
                  (m_ypos[iFirst] == ship.m_ypos[iSecond]))     ||
                 ((m_xpos[iFirst]+1 == ship.m_xpos[iSecond]) &&
                  (m_ypos[iFirst] == ship.m_ypos[iSecond]))     ||
                 ((m_xpos[iFirst] == ship.m_xpos[iSecond]) &&
                  (m_ypos[iFirst]+1 == ship.m_ypos[iSecond])) )
                return 1;
    return 0;
}

//-------------------------------------------------------------------------

BigShip::BigShip(int length, int init_x, int init_y, int xdir, int ydir)
        : Ship(length, init_x, init_y, xdir, ydir), m_numberOfHits(0)
{
}

//-------------------------------------------------------------------------

int BigShip::hit(int xtarget, int ytarget)
{
    if (m_sunk == 1) return 0;
    for (int i=0; i<m_length; i++)
        if ((m_xpos[i]==xtarget)&&(m_ypos[i]==ytarget))
        {
            m_xpos[i] = m_xpos[i] + 100; // mark this position
            m_ypos[i] = m_ypos[i] + 100;
            m_numberOfHits++;
            if (m_numberOfHits >= 2) m_sunk = 1;
            return 1;
        }
    return 0;
}

//-------------------------------------------------------------------------

char TwoDArray::invalid = 'I';

TwoDArray::TwoDArray(int xdim, int ydim)
{
    if ((xdim > 0) && (ydim > 0))
    {
        m_xdim = xdim;
        m_ydim = ydim;
        elems = new char[xdim*ydim];
        for (int i=0; i<xdim*ydim; i++) elems[i] = ' ';
    }
    else
        TRACE("failed to construct TwoDArray::%d,%d\n",xdim,ydim);
}

TwoDArray::~TwoDArray()
{
    delete[] elems;
}

char& TwoDArray::elemAt(int x_index, int y_index)
{
    if (elems != 0)
    {
        if ( ((x_index >= 0) && (x_index < m_xdim)) &&
             ((y_index >= 0) && (y_index < m_ydim)) )
            return elems[y_index*m_xdim+x_index];
        else
            TRACE("TwoDArray access out of bound:: %d,%d\n",x_index,
                   y_index);
    }
    else
        TRACE("use of an improperly constructed TwoDArray\n");
    return invalid;
}

TwoDArray::TwoDArray(const TwoDArray &src)
{
    m_xdim = src.m_xdim;
    m_ydim = src.m_ydim;
    elems = new char[m_xdim*m_ydim];
    for (int i=0; i<m_xdim*m_ydim; i++)
        elems[i] = src.elems[i];
}

//-------------------------------------------------------------------------

Random::Random()
{
    srand((int) time(NULL));
}

int Random::next(int low, int high)
{
    int k;
    double d;
    d = (double) rand() / ((double) RAND_MAX + 1);
    k = (int) (d * (high - low + 1));
    return (low + k);
}

